@@ -5,7 +5,7 @@
 
     This module implements various URL related functions.
 
-    :copyright: (c) 2013 by the Werkzeug Team, see AUTHORS for more details.
+    :copyright: (c) 2014 by the Werkzeug Team, see AUTHORS for more details.
     :license: BSD, see LICENSE for more details.
 """
 import re
@@ -312,7 +312,7 @@ def _url_encode_impl(obj, charset, encode_keys, sort, key):
             key = text_type(key).encode(charset)
         if not isinstance(value, bytes):
             value = text_type(value).encode(charset)
-        yield url_quote(key) + '=' + url_quote_plus(value)
+        yield url_quote_plus(key) + '=' + url_quote_plus(value)
 
 
 def _url_unquote_legacy(value, unsafe=''):
@@ -371,24 +371,32 @@ def url_parse(url, scheme=None, allow_fragments=True):
     return result_type(scheme, netloc, url, query, fragment)
 
 
-def url_quote(string, charset='utf-8', errors='strict', safe='/:'):
+def url_quote(string, charset='utf-8', errors='strict', safe='/:', unsafe=''):
     """URL encode a single string with a given encoding.
 
     :param s: the string to quote.
     :param charset: the charset to be used.
     :param safe: an optional sequence of safe characters.
+    :param unsafe: an optional sequence of unsafe characters.
+
+    .. versionadded:: 0.9.2
+       The `unsafe` parameter was added.
     """
+    if not isinstance(string, (text_type, bytes, bytearray)):
+        string = text_type(string)
     if isinstance(string, text_type):
         string = string.encode(charset, errors)
     if isinstance(safe, text_type):
         safe = safe.encode(charset, errors)
-    safe = frozenset(bytearray(safe) + _always_safe)
+    if isinstance(unsafe, text_type):
+        unsafe = unsafe.encode(charset, errors)
+    safe = frozenset(bytearray(safe) + _always_safe) - frozenset(bytearray(unsafe))
     rv = bytearray()
     for char in bytearray(string):
         if char in safe:
             rv.append(char)
         else:
-            rv.extend(('%%%X' % char).encode('ascii'))
+            rv.extend(('%%%02X' % char).encode('ascii'))
     return to_native(bytes(rv))
 
 
@@ -400,7 +408,7 @@ def url_quote_plus(string, charset='utf-8', errors='strict', safe=''):
     :param charset: The charset to be used.
     :param safe: An optional sequence of safe characters.
     """
-    return url_quote(string, charset, errors, safe + ' ').replace(' ', '+')
+    return url_quote(string, charset, errors, safe + ' ', '+').replace(' ', '+')
 
 
 def url_unparse(components):
@@ -514,9 +522,9 @@ def uri_to_iri(uri, charset='utf-8', errors='replace'):
     if isinstance(uri, tuple):
         uri = url_unparse(uri)
     uri = url_parse(to_unicode(uri, charset))
-    path = url_unquote(uri.path, charset, errors, '/;?')
-    query = url_unquote(uri.query, charset, errors, ';/?:@&=+,$')
-    fragment = url_unquote(uri.fragment, charset, errors, ';/?:@&=+,$')
+    path = url_unquote(uri.path, charset, errors, '%/;?')
+    query = url_unquote(uri.query, charset, errors, '%;/?:@&=+,$')
+    fragment = url_unquote(uri.fragment, charset, errors, '%;/?:@&=+,$')
     return url_unparse((uri.scheme, uri.decode_netloc(),
                         path, query, fragment))
 
